local class = select(2, UnitClass("player"))
local classcolor = RAID_CLASS_COLORS[class]

local setup = {
	width = 200,
	height = 18,
	scale = 1,
}

local classSpells = {
	WARRIOR = {
		469, -- Commanding Shout
		6673, -- Battle Shout
		7386, -- Sunder Armor
		6343, -- Thunder Clap
		1160, -- Demoralizing Shout
		{ -- Rend
		spell = 772,
		player = true,
		},
	},
	PRIEST =
	{  { -- Shadow Word: Pain
		spell = 48125,
		player = true,
	}, { -- Abolish Disease
		spell = 552,
		player = true,
	}, { -- Renew
		spell = 48068,
		player = true,
	}, { -- Devouring Plague
		spell = 48300,
		player = true,
	}, { -- Vampiric Embrace
		spell = 15286,
		player = true,
	}, { -- Vampiric Touch
		spell = 48160,
		player = true,
	},
		33206, -- Pain Suppression
		10060, -- Power Infusion
		47788, -- Guardian Spirit
		453,   -- Mind Soothe
		10955, -- Shackle Undead
	},
	WARLOCK =
	{  { -- Curse of Doom
		spell = 47867,
		player = true,
	}, { -- Immolate
		spell = 47811,
		player = true,
	}, },
}

local bars = {}
local function createFrame(data)
	local spellId = type(data) == "table" and data.spell or data
	local playerOnly = type(data) == "table" and data.player or false
	local spellName, _, spellIcon = GetSpellInfo(spellId)
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:SetWidth(setup.width)
	frame:SetHeight(setup.height)
	frame:SetScale(setup.scale)
	frame:SetMovable(1)

	local icon = frame:CreateTexture(nil, "BACKGROUND")
	icon:SetPoint("TOPLEFT", frame)
	icon:SetHeight(setup.height)
	icon:SetWidth(setup.height)
	icon:SetTexture(spellIcon)
	icon:SetTexCoord(0.07, 0.93, 0.07, 0.93) 

	local statusbar = CreateFrame("StatusBar", nil, frame)
	statusbar:SetPoint("TOPLEFT", icon, "TOPRIGHT")
	statusbar:SetWidth(setup.width - setup.height)
	statusbar:SetHeight(setup.height)
	statusbar:SetStatusBarTexture("Interface\\AddOns\\JustTimer\\statusbar")
	statusbar:SetStatusBarColor(classcolor.r, classcolor.g, classcolor.b, 1)
	statusbar:SetMinMaxValues(0, 1)
	statusbar:SetValue(0)
	
	local bg = statusbar:CreateTexture(nil, "BACKGROUND")
	bg:SetAllPoints()
	bg:SetTexture("Interface\\AddOns\\JustTimer\\statusbar")
	bg:SetVertexColor(0.5, 0.5, 0.5, 0.7)

	local time = statusbar:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmallOutline")
	time:SetPoint("RIGHT", statusbar, -2, 0)

	local name = statusbar:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmallOutline")
	name:SetAllPoints(frame)
	name:SetJustifyH("CENTER")
	name:SetJustifyV("MIDDLE")
	name:SetText(spellName)

	frame.statusbar = statusbar
	frame.time = time
	frame.playerOnly = playerOnly
	
	frame:Hide()
	bars[spellName] = frame
end

local function rearrangeBars()
	local lastBar = nil
	for spell, bar in pairs(bars) do
		if bar:IsShown() then
			if lastBar then
				bar:SetPoint("BOTTOMLEFT", lastBar, "TOPLEFT")
			else
				if oUF_Player then
					bar:SetPoint("BOTTOMLEFT", oUF_Player, "TOPLEFT", 0, 4)
				else
					bar:SetPoint("CENTER", UIParent, "CENTER", 0, -200)
				end
			end
			lastBar = bar
		end
	end
end

local function stop(bar)
	bar:SetScript("OnUpdate", nil)
	bar:Hide()
	rearrangeBars()
end

local function onUpdate(self)
	local t = GetTime()
	if t >= self.exp or not UnitExists(self.unit) then
		stop(self)
	else
		local time = self.exp - t
		self.statusbar:SetValue(time)
		self.time:SetFormattedText(SecondsToTimeAbbrev(time))
	end
end

local function start(bar, unit, duration, expirationTime)
	bar.statusbar:SetMinMaxValues(0, duration)
	bar.exp = expirationTime
	bar.unit = unit
	bar:SetScript("OnUpdate", onUpdate)
	bar:Show()
end

do
	local auraScanner = CreateFrame("Frame")
	local currentScan = {}
	auraScanner:SetScript("OnEvent", function(self, event, unit)
		local u = unit or (event == "PLAYER_TARGET_CHANGED" and "target" or "player")
		if event == "PLAYER_TARGET_CHANGED" and not UnitExists("target") then u = "player" end
		if u ~= "player" and u ~= "target" then return end
		for i = 1, 40 do
			local name, _, _, _, _, duration, expirationTime, caster = UnitAura(u, i, "HELPFUL")
			if not name then break end
			currentScan[name] = true
			if bars[name] then
				local bar = bars[name]
				if (bar.playerOnly and caster == "player" or not bar.playerOnly) and duration > 0 then
					start(bar, u, duration, expirationTime)
				end
			end
		end
		for i = 1, 40 do
			local name, _, _, _, _, duration, expirationTime, caster = UnitAura(u, i, "HARMFUL")
			if not name then break end
			currentScan[name] = true
			if bars[name] then
				local bar = bars[name]
				if (bar.playerOnly and caster == "player" or not bar.playerOnly) and duration > 0 then
					start(bar, u, duration, expirationTime)
				end
			end
		end
		for name, bar in pairs(bars) do
			if bar.unit == u and not currentScan[name] then stop(bar) end
		end
		wipe(currentScan)
		rearrangeBars()
	end)
	auraScanner:RegisterEvent("UNIT_AURA")
	auraScanner:RegisterEvent("PLAYER_TARGET_CHANGED")
	auraScanner:RegisterEvent("PLAYER_ENTERING_WORLD")
end
local addon = CreateFrame("Frame")
addon:SetScript("OnEvent", function(self, event, ...)
	if event == "VARIABLES_LOADED" then
		if oUF_Player then
			setup.width = oUF_Player:GetWidth()
		end
		if classSpells[class] then
			for c in pairs(classSpells) do
				if c ~= class then classSpells[c] = nil end
			end
			for i = 1, #classSpells[class] do
				createFrame(classSpells[class][i])
			end
		end
	end
end)
addon:RegisterEvent("VARIABLES_LOADED")

